<?php

namespace Drupal\global_node_translation\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\Node;
use Drupal\views_bulk_operations\Action\ViewsBulkOperationsActionBase;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Views Bulk Operations (VBO) action for translating nodes.
 *
 * @Action(
 *   id = "global_node_translation_translate_nodes",
 *   label = @Translation("Translate Nodes"),
 *   type = "node"
 * )
 */
class TranslateNodes extends ViewsBulkOperationsActionBase implements ContainerFactoryPluginInterface {

  /**
   * The translation service.
   *
   * @var \Drupal\global_node_translation\Service\TranslationService
   */
  protected $translationService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new TranslateNodes object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\global_node_translation\Service\TranslationService $translation_service
   *   The translation service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, $translation_service, MessengerInterface $messenger, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->translationService = $translation_service;
    $this->messenger = $messenger;
    $this->logger = $logger;
  }

  /**
   * Creates an instance of the TranslateNodes action.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return \Drupal\global_node_translation\Plugin\Action\TranslateNodes
   *   The created action instance.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('global_node_translation.translation_service'),
      $container->get('messenger'),
      $container->get('logger.factory')->get('global_node_translation')
    );
  }

  /**
   * Executes the action for the given node.
   *
   * This method is called when the action is performed. It translates the node
   * and adds a success message or logs an error if the translation fails.
   *
   * @param \Drupal\node\Entity\Node|null $entity
   *   The node entity to translate.
   */
  public function execute($entity = NULL) {
    if ($entity instanceof Node) {
      $content_type_id = $entity->bundle();

      // Retrieve the configuration for the specific content type.
      $config = \Drupal::config('global_node_translation.settings' . $content_type_id);
      $automatic_translation_enabled = $config->get('enable_automatic_translation');

      if ($automatic_translation_enabled) {
        try {
          $this->translationService->translateCreateTranslations($entity);
          $this->messenger->addMessage($this->t('Selected nodes are translated successfully'));
        }
        catch (\Exception $e) {
          $this->messenger->addError($this->t('An error occurred while creating translations for node @nid. Error: @message', [
            '@nid' => $entity->id(),
            '@message' => $e->getMessage(),
          ]));
        }
      }
      else {
        $this->messenger->addError($this->t('Automatic translation is not enabled for the content type "@type".
        No translation was performed.', ['@type' => $content_type_id]));
      }
    }
  }

  /**
   * Determines access to the action.
   *
   * Checks whether the current user has permission to administer nodes and
   * whether the entity is a valid node.
   *
   * @param mixed $object
   *   The object to check access for.
   * @param \Drupal\Core\Session\AccountInterface|null $account
   *   The user account.
   * @param bool $return_as_object
   *   Whether to return the result as an object or a boolean.
   *
   * @return \Drupal\Core\Access\AccessResult|bool
   *   Access result or boolean value indicating access.
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $result = $object instanceof Node && $account->hasPermission('administer nodes');
    if ($return_as_object) {
      return AccessResult::allowedIf($result);
    }
    return $result;
  }

  /**
   * Builds the configuration form for the action.
   *
   * Provides a form for configuring the action. Currently, no configuration
   * form fields are available for this action.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form array.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * Submits the configuration form for the action.
   *
   * Handles form submission for action configuration. Currently, no form fields
   * need to be submitted.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // No configuration form fields to submit.
  }

}
