<?php

namespace Drupal\global_node_translation\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\node\Entity\Node;
use Drupal\paragraphs\Entity\Paragraph;
use Stichoza\GoogleTranslate\GoogleTranslate;

/**
 * Service for translating text and nodes.
 */
class GlobalNodeTranslationService {

  /**
   * The logger channel factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a TranslationService object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    EntityTypeManagerInterface $entity_type_manager,
    LanguageManagerInterface $language_manager,
    ConfigFactoryInterface $config_factory
  ) {
    $this->logger = $logger_factory->get('global_node_translation');
    $this->entityTypeManager = $entity_type_manager;
    $this->languageManager = $language_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Translates text using the Google Translate library.
   *
   * @param string $text
   *   The text to translate.
   * @param string $targetLang
   *   The target language code.
   *
   * @return string
   *   The translated text.
   */
  public function translateText($text, $targetLang) {
    if (empty($text)) {
      return $text;
    }

    try {
      $tr = new GoogleTranslate();
      $tr->setSource();
      $tr->setTarget($targetLang);
      return $tr->translate($text);
    }
    catch (\Exception $e) {
      $this->logger->error('Translation failed: @message', ['@message' => $e->getMessage()]);
      // Return original text if translation fails.
      return $text;
    }
  }

  /**
   * Check if automatic translation is enabled for a given content type.
   */
  public function isAutomaticTranslationEnabled(Node $node) {
    $content_type_id = $node->bundle();
    $config = $this->configFactory->get('global_node_translation.settings' . $content_type_id);
    return $config->get('enable_automatic_translation');
  }

  /**
   * Creates translations of a node for all enabled languages.
   */
  public function translateCreateTranslations(Node $node) {

    if (!$this->isAutomaticTranslationEnabled($node)) {
      return;
    }

    $config = $this->configFactory->get('global_node_translation.settings');
    $enabled_languages = $config->get('enabled_languages');

    foreach ($enabled_languages as $langcode => $language) {
      if ($langcode !== $node->get('langcode')->value) {
        if (!$node->hasTranslation($langcode)) {
          $translated_node = $node->addTranslation($langcode, $node->toArray());
          $this->translateFields($node, $translated_node, $langcode);
          $translated_node->save();
        }
      }
    }
  }

  /**
   * Translates fields of a node.
   */
  protected function translateFields(Node $node, Node $translated_node, $langcode) {
    foreach ($node->getFieldDefinitions() as $field_name => $field_definition) {
      if ($field_definition->isTranslatable()) {
        $field = $node->get($field_name);
        $translated_field = $translated_node->get($field_name);

        if (in_array($field_definition->getType(), ['text', 'text_long', 'text_with_summary', 'string', 'string_long'])) {
          foreach ($field as $delta => $item) {
            if (!is_null($item->value)) {
              $translated_value = $this->translateText($item->value, $langcode);
              $translated_field->get($delta)->value = $translated_value;
              // dd($translated_value);
            }
            // Check for text_with_summary to translate the summary field.
            if ($field_definition->getType() === 'text_with_summary' && !is_null($item->summary)) {
              $translated_summary = $this->translateText($item->summary, $langcode);
              $translated_field->get($delta)->summary = $translated_summary;
            }
          }
        }
        elseif ($field_definition->getType() === 'link') {
          foreach ($field as $delta => $item) {
            $translated_title = $this->translateText($item->title, $langcode);
            // URL remains the same.
            $translated_field->get($delta)->uri = $item->uri;
            $translated_field->get($delta)->title = $translated_title;
          }
        }
        elseif ($field_definition->getType() === 'entity_reference_revisions' && $field_definition->getSetting('target_type') === 'paragraph') {
          foreach ($field as $delta => $item) {
            if ($item->entity instanceof Paragraph) {
              $translated_paragraph = $this->translateCreateParagraphTranslation($item->entity, $langcode);
              $translated_field->get($delta)->target_id = $translated_paragraph->id();
            }
          }
        }
      }
    }
  }

  /**
   * Creates a translation for a paragraph.
   */
  protected function translateCreateParagraphTranslation(Paragraph $paragraph, $langcode) {
    if (!$paragraph->hasTranslation($langcode)) {
      // Create a new translation.
      $translated_paragraph = $paragraph->addTranslation($langcode, $paragraph->toArray());
      // Translate fields in the new translation.
      $this->translateParagraphFields($paragraph, $translated_paragraph, $langcode);
      // Save the new translation.
      $translated_paragraph->save();
      return $translated_paragraph;
    }
    return $paragraph->getTranslation($langcode);
  }

  /**
   * Translates fields of a paragraph.
   */
  protected function translateParagraphFields(Paragraph $paragraph, Paragraph $translated_paragraph, $langcode) {
    foreach ($paragraph->getFieldDefinitions() as $field_name => $field_definition) {
      if ($field_definition->isTranslatable()) {
        $field = $paragraph->get($field_name);
        $translated_field = $translated_paragraph->get($field_name);

        // Translate text fields.
        if (in_array($field_definition->getType(), ['text', 'text_long', 'text_with_summary', 'string', 'string_long'])) {
          foreach ($field as $delta => $item) {
            if (!is_null($item->value)) {
              // Translate the field value.
              $translated_value = $this->translateText($item->value, $langcode);
              $translated_field->get($delta)->value = $translated_value;
            }
            // Check for text_with_summary to translate the summary field.
            if ($field_definition->getType() === 'text_with_summary' && !is_null($item->summary)) {
              $translated_summary = $this->translateText($item->summary, $langcode);
              $translated_field->get($delta)->summary = $translated_summary;
            }
          }
        }
        // Translate link fields.
        elseif ($field_definition->getType() === 'link') {
          foreach ($field as $delta => $item) {
            // Translate the link title.
            $translated_field->get($delta)->uri = $item->uri;
            $translated_field->get($delta)->title = $this->translateText($item->title, $langcode);
          }
        }

        // Translate entity reference fields.
        elseif ($field_definition->getType() === 'entity_reference_revisions' && $field_definition->getSetting('target_type') === 'paragraph') {
          foreach ($field as $delta => $item) {
            if ($item->entity instanceof Paragraph) {
              // Translate the referenced paragraph.
              $translated_referenced_paragraph = $this->translateCreateParagraphTranslation($item->entity, $langcode);
              $translated_field->get($delta)->target_id = $translated_referenced_paragraph->id();
              $translated_field->get($delta)->target_revision_id = $translated_referenced_paragraph->getRevisionId();
            }
          }
        }
      }
    }
  }

}
